#include <stdlib.h>
#include "curses.h"
#include "term.h"

#ifdef QDOS
#include <qdos.h>
char _prog_name[] = "infocmp";
char _version[]   = " v1.00 ";
char _copyright[]  = "(c)1991-1994  K.D.Walker";
void (*_consetup)() = consetup_title;           /* Fancy titled screen */
#endif /* QDOS */

#define LIST_TERMINFO 0
#define LIST_VARIABLE 1
#define LIST_TERMCAP  2

#define DIFF_DIFFERENCE 0
#define DIFF_COMMON     1
#define DIFF_NEITHER    2

extern int optind;
extern char *optarg;
extern int getopt();
extern int putenv();

static int  list_format = LIST_TERMINFO;
static int  diff_format = DIFF_DIFFERENCE;
static char *A_directory = (char *)NULL;
static char *B_directory = (char *)NULL;

static void
list_term(TERMINAL *term, int format)
{
  char *sbptr, *ebptr;
  short *snptr, *enptr;
  char **ssptr, **esptr;
  char **nptr;

  format = format;
  printf("%s,\n", term->_termdesc);

  for(sbptr = &(term->_bool_starter) + 1,
      ebptr = &(term->_bool_terminator),
      nptr = boolnames;
      sbptr != ebptr;
      sbptr++, nptr++)
    {
    if(*sbptr == 1) printf("\t%s\n",*nptr);
    }

  for(snptr = &(term->_numeric_starter) + 1,
      enptr = &(term->_numeric_terminator),
      nptr = numnames;
      snptr != enptr;
      snptr++, nptr++)
    {
    if(*snptr >= 0) printf("\t%s#%d,\n", *nptr, *snptr);
    }

  for(ssptr = &(term->_string_starter) + 1,
      esptr = &(term->_string_terminator),
      nptr = strnames;
      ssptr != esptr;
      ssptr++, nptr++)
    {
    if(*ssptr)
      {
      char *s;

      printf("\t%s=", *nptr);
      for(s = *ssptr; *s; s++) printf("%s", unctrl(*s));
      printf(",\n");
      }
    }
}


static void
compare_term(TERMINAL *term1, TERMINAL *term2, int format)
{
  char *sbptr, *ebptr, *bptr2;
  short *snptr, *enptr, *nptr2;
  char **ssptr, **esptr, **sptr2;
  char **nptr;

  printf("%s:%s,\n", term1->_termdesc, term2->_termdesc);

  for(sbptr = &(term1->_bool_starter) + 1,
      ebptr = &(term1->_bool_terminator),
      bptr2 = &(term2->_bool_starter) + 1,
      nptr = boolnames;
      sbptr != ebptr;
      sbptr++, nptr++)
    {
    if(*sbptr == *bptr2)
      {
      if(*sbptr == 1 && format == DIFF_COMMON)
        printf("\t%s=%c\n",*nptr, *sbptr?'T':'F');
      }
    else
      {
      if(format == DIFF_DIFFERENCE)
        printf("\t%s=%c:%c\n", *nptr,*sbptr?'T':'F',*bptr2?'T':'F');
      }
    }

  for(snptr = &(term1->_numeric_starter) + 1,
      enptr = &(term1->_numeric_terminator),
      nptr2 = &(term2->_numeric_starter) + 1,
      nptr = numnames;
      snptr != enptr;
      snptr++, nptr++)
    {
    printf("\t%s=%d:%d\n", *nptr,(*snptr<0)?-1:*snptr,(*nptr2<0)?-1:*nptr2);
    if(*snptr == *nptr2)
      {
      if(format == DIFF_COMMON)
        printf("\t%s=%d\n", *nptr, (*snptr<0)?-1:*snptr);
      }
    else
      {
      if(format == DIFF_DIFFERENCE)
        printf("\t%s=%d:%d\n", *nptr,
          (*snptr<0)?-1:*snptr, (*nptr2<0)?-1:*nptr2);
      }
    }

  for(ssptr = &(term1->_string_starter) + 1,
      esptr = &(term1->_string_terminator),
      sptr2 = &(term2->_string_starter) + 1,
      nptr = strnames;
      ssptr != esptr;
      ssptr++, nptr++)
    {
    }
}

int
main(int argc, char *argv[])
{
  int c;
  char *termnam = (char *)0;
  char buffer[256];
  TERMINAL *term1;
  TERMINAL *term2;

  while((c = getopt(argc, argv, "1cdnrs:uvw:A:b:ILCV")) != EOF)
    {
    switch(c)
      {
    /* options for comparing 2 terminfo files */
    case 'c' : diff_format = DIFF_COMMON; break;
    case 'd' : diff_format = DIFF_DIFFERENCE; break;
    case 'n' : diff_format = DIFF_NEITHER; break;

    /* options for listing a terminfo description */
    case 'I' : list_format = LIST_TERMINFO; break;
    case 'L' : list_format = LIST_VARIABLE; break;
    case 'C' : list_format = LIST_TERMCAP; break;
    case 'r' : break;

    /* option for use= option */
    case 'u' : break;

    /* Directory setting option */
    case 'A' : A_directory = optarg; break;
    case 'B' : B_directory = optarg; break;

    case 's' : break;
    case 'v' : break;
    case 'w' : break;
    case '1' : break;
    case 'V' : break;

    case '?' :
      fprintf(stderr,"usage: %s [-cdnrvILCV] [-s d|i|l|c] [-A directory] [-B directory] [-w width] [termname ...]\n", argv[0]);
      exit (1);
      }
    }

  if(A_directory)
    {
    sprintf(buffer, "TERMINFO=%s", A_directory);
    putenv(buffer);
    }
  if(optind < argc) termnam = argv[optind++];

  setupterm(termnam, 1, (int *)0);
  term1 = cur_term;

  if(optind == argc)
    {
    list_term(cur_term, list_format);
    exit (1);
    }

  termnam = argv[optind];
  if(B_directory)
    {
    sprintf(buffer, "TERMINFO=%s", B_directory);
    putenv(buffer);
    }
  setupterm(termnam, 1, (int *) 0);
  term2 = cur_term;

  compare_term(term1, term2, diff_format);
  return(0);
}
